"""The Room class."""

import logging

from tuya_vacuum.utils import (
    chunks,
    combine_high_low_to_int,
    hex_to_ints,
)

MAX_ID = 255  # Max id number
INFO_BYTE_LEN = 26  # "Room properties"
NAME_BYTE_LEN = 20  # "Vertices_name"

_LOGGER = logging.getLogger(__name__)


class Room:
    """A room inside the layout map of a home generated by a vacuum cleaner."""

    def __init__(self, data: str, byte_pos: int) -> None:
        """Parse the data of a room in the vacuum map.

        @param data: The `map_room_array`.
        @param byte_pos: The byte position of the room in the `map_room_array`.
        """
        _LOGGER.debug("Parsing room")

        # "room information" according to google translate
        room_info_str = data[
            byte_pos : (byte_pos + (INFO_BYTE_LEN + NAME_BYTE_LEN + 1) * 2)
        ]

        [self.id, self.order, self.sweep_count, self.mop_count] = [
            combine_high_low_to_int(integer[0], integer[1])
            for integer in chunks(hex_to_ints(room_info_str[:16]), 2)
        ]

        [
            self.color_order,
            self.sweep_forbidden,
            self.mop_forbidden,
            self.fan,
            self.water_level,
            self.y_mode,
        ] = hex_to_ints(room_info_str[16:28])

        self.name_length = int(room_info_str[52:54], 16)
        vertices_name_str = room_info_str[
            (INFO_BYTE_LEN * 2 + 1 * 2) : (
                INFO_BYTE_LEN * 2 + 1 * 2 + self.name_length * 2
            )
        ]

        self.name = bytes.fromhex(vertices_name_str).decode()

        self.vertex_num = int(room_info_str[-2:], 16)
        self.vertex_str = data[
            (byte_pos + (INFO_BYTE_LEN + NAME_BYTE_LEN + 1) * 2) : (
                byte_pos
                + (INFO_BYTE_LEN + NAME_BYTE_LEN + 1) * 2
                + self.vertex_num * 2 * 2 * 2
            )
        ]

        _LOGGER.debug("Finished parsing room")

    @staticmethod
    def parse_rooms(data: str) -> list["Room"]:
        """Parse the map room array.

        @param data: The `map_room_array`.
        """
        _LOGGER.debug("Parsing rooms")

        rooms = []
        room_count = int(data[2:4], 16)

        _LOGGER.debug("Found %s rooms", room_count)

        byte_pos = 4
        for index in range(room_count):
            _LOGGER.debug("Parsing room %s", index)

            # Parse the data into a new room and add it to list
            rooms.append(Room(data, byte_pos))
            # Move to the start of the next room's data
            byte_pos += (INFO_BYTE_LEN + NAME_BYTE_LEN + 1) * 2

        return rooms
